@@ -20,7 +20,6 @@
 #include <uct/api/v2/uct_v2.h>
 #include <ucs/datastruct/mpool.h>
 #include <ucs/datastruct/queue_types.h>
-#include <ucs/datastruct/bitmap.h>
 #include <ucs/datastruct/conn_match.h>
 #include <ucs/memory/memtype_cache.h>
 #include <ucs/memory/memory_type.h>
@@ -47,8 +46,10 @@ enum {
 typedef struct ucp_context_config {
     /** Threshold for switching UCP to buffered copy(bcopy) protocol */
     size_t                                 bcopy_thresh;
-    /** Threshold for switching UCP to rendezvous protocol */
-    size_t                                 rndv_thresh;
+    /** Threshold for switching UCP to rendezvous protocol for intra-node */
+    size_t                                 rndv_intra_thresh;
+    /** Threshold for switching UCP to rendezvous protocol for inter-node */
+    size_t                                 rndv_inter_thresh;
     /** Threshold for switching UCP to rendezvous protocol
      *  in ucp_tag_send_nbr() */
     size_t                                 rndv_send_nbr_thresh;
@@ -166,6 +167,15 @@ typedef struct ucp_context_config {
     uint64_t                               reg_nb_mem_types;
     /** Prefer native RMA transports for RMA/AMO protocols */
     int                                    prefer_offload;
+    /** RMA zcopy segment size */
+    size_t                                 rma_zcopy_max_seg_size;
+    /** Protocol overhead */
+    double                                 proto_overhead_single;
+    double                                 proto_overhead_multi;
+    double                                 proto_overhead_rndv_offload;
+    double                                 proto_overhead_rndv_rts;
+    double                                 proto_overhead_rndv_rtr;
+    double                                 proto_overhead_sw;
 } ucp_context_config_t;
 
 
@@ -268,6 +278,14 @@ typedef struct ucp_tl_md {
 } ucp_tl_md_t;
 
 
+typedef struct ucp_context_alloc_md_index {
+    int            initialized;
+    /* Index of memory domain that is used to allocate memory of the given type
+     * using ucp_memh_alloc(). */
+    ucp_md_index_t md_index;
+} ucp_context_alloc_md_index_t;
+
+
 /**
  * UCP context
  */
@@ -278,15 +296,19 @@ typedef struct ucp_context {
     ucp_tl_md_t                   *tl_mds;    /* Memory domain resources */
     ucp_md_index_t                num_mds;    /* Number of memory domains */
 
-    /* Index of memory domain that is used to allocate memory of the given type
-     * using ucp_memh_alloc(). */
-    int                           alloc_md_index_initialized;
-    ucp_md_index_t                alloc_md_index[UCS_MEMORY_TYPE_LAST];
+    ucp_context_alloc_md_index_t  alloc_md[UCS_MEMORY_TYPE_LAST];
 
     /* Map of MDs that provide registration for given memory type,
        ucp_mem_map() will register memory for all those domains. */
     ucp_md_map_t                  reg_md_map[UCS_MEMORY_TYPE_LAST];
 
+    /* Map of MDs that provide blocking registration for given memory type.
+     * This map is initialized if non-blocking registration is requested for
+     * the memory type (thus reg_md_map contains only MDs supporting
+     * non-blocking registration).
+     */
+    ucp_md_map_t                  reg_block_md_map[UCS_MEMORY_TYPE_LAST];
+
     /* Map of MDs that require caching registrations for given memory type. */
     ucp_md_map_t                  cache_md_map[UCS_MEMORY_TYPE_LAST];
 
@@ -505,12 +527,13 @@ typedef struct ucp_tl_iface_atomic_flags {
         const uct_md_attr_v2_t *md_attr; \
         ucp_md_index_t md_index; \
         ucp_rsc_index_t tl_id; \
-        UCS_BITMAP_CLEAR(&(_tl_bitmap)); \
-        UCS_BITMAP_FOR_EACH_BIT((_context)->tl_bitmap, tl_id) { \
+        \
+        UCS_STATIC_BITMAP_RESET_ALL(&(_tl_bitmap)); \
+        UCS_STATIC_BITMAP_FOR_EACH_BIT(tl_id, &(_context)->tl_bitmap) { \
             md_index = (_context)->tl_rscs[tl_id].md_index; \
             md_attr  = &(_context)->tl_mds[md_index].attr; \
             if (md_attr->_cap_field & UCS_BIT(_mem_type)) { \
-                UCS_BITMAP_SET(_tl_bitmap, tl_id); \
+                UCS_STATIC_BITMAP_SET(&(_tl_bitmap), tl_id); \
             } \
         } \
     }
