@@ -435,6 +435,39 @@ ucs_status_t ucs_sock_ipstr_to_sockaddr(const char *ip_str,
                                         struct sockaddr_storage *sa_storage);
 
 
+/**
+ * Extract the IP address and port from a given string and return it as a
+ * sockaddr storage. The supported formats are (as per RFCs 3986, 5952):
+ * IPv4:port,   e.g. 127.0.0.1:1338
+ * [IPv6]:port  e.g. [::1]:1339
+ *
+ * Port is optional parameter in string, if not set, default_port is used
+ *
+ * @param [in]  ip_port_str  A string to take IP address and port from.
+ * @param [in]  default_port Default port to use if string doesn't have it.
+ * @param [out] sa_storage   sockaddr storage filled with the IP address,
+ *                           address family and port.
+ *
+ * @return UCS_OK if @a ip_port_str has a valid IP address,
+ *         UCS_ERR_INVALID_ADDR otherwise.
+ */
+ucs_status_t ucs_sock_ipportstr_to_sockaddr(const char *ip_port_str,
+                                            uint16_t default_port,
+                                            struct sockaddr_storage *sa_storage);
+
+
+/**
+ * Extract port from a given string and validate it.
+ *
+ * @param [in]  port_str A string to take port from.
+ * @param [out] port     Variable to store parsed port.
+ *
+ * @return UCS_OK if @a port_str has a valid port,
+ *         UCS_ERR_INVALID_ADDR otherwise.
+ */
+ucs_status_t ucs_sock_port_from_string(const char *port_str, uint16_t *port);
+
+
 /**
  * Check if the address family of the given sockaddr is IPv4 or IPv6
  *
