@@ -201,6 +201,26 @@ ucs_config_field_t uct_ib_iface_config_table[] = {
    "detect the default value by creating a dummy QP." ,
    ucs_offsetof(uct_ib_iface_config_t, counter_set_id), UCS_CONFIG_TYPE_ULUNITS},
 
+  {"REVERSE_SL", "auto",
+   "Reverse Service level. 'auto' will set the same value of sl\n",
+   ucs_offsetof(uct_ib_iface_config_t, reverse_sl), UCS_CONFIG_TYPE_ULUNITS},
+
+  {"SEND_OVERHEAD", UCS_VALUE_AUTO_STR,
+   "Estimated overhead of preparing a work request, posting it to the NIC,\n"
+   "and finalizing an operation",
+   0, UCS_CONFIG_TYPE_KEY_VALUE(UCS_CONFIG_TYPE_TIME_UNITS,
+    {"bcopy", "estimated overhead of allocating a tx buffer",
+     ucs_offsetof(uct_ib_iface_config_t, send_overhead.bcopy)},
+    {"cqe", "estimated overhead of processing a work request completion",
+     ucs_offsetof(uct_ib_iface_config_t, send_overhead.cqe)},
+    {"db", "estimated overhead of writing a doorbell to PCI",
+     ucs_offsetof(uct_ib_iface_config_t, send_overhead.db)},
+    {"wqe_fetch", "estimated overhead of fetching a wqe",
+     ucs_offsetof(uct_ib_iface_config_t, send_overhead.wqe_fetch)},
+    {"wqe_post", "estimated overhead of posting a wqe",
+     ucs_offsetof(uct_ib_iface_config_t, send_overhead.wqe_post)},
+    {NULL})},
+
   {NULL}
 };
 
@@ -716,7 +736,7 @@ static int uct_ib_iface_gid_extract_flid(const union ibv_gid *gid)
     return ntohs(*((uint16_t*)UCS_PTR_BYTE_OFFSET(gid->raw, 4)));
 }
 
-static int uct_ib_iface_is_flid_enabled(const uct_ib_iface_t *iface)
+static int uct_ib_iface_is_flid_enabled(uct_ib_iface_t *iface)
 {
     return iface->config.flid_enabled &&
            (uct_ib_iface_gid_extract_flid(&iface->gid_info.gid) != 0);
@@ -850,7 +870,7 @@ static uint16_t uct_ib_gid_site_local_subnet_prefix(const union ibv_gid *gid)
     return be64toh(gid->global.subnet_prefix) & 0xffff;
 }
 
-uint16_t uct_ib_iface_resolve_remote_flid(const uct_ib_iface_t *iface,
+uint16_t uct_ib_iface_resolve_remote_flid(uct_ib_iface_t *iface,
                                           const union ibv_gid *gid)
 {
     if (!uct_ib_iface_is_flid_enabled(iface)) {
@@ -1088,9 +1108,9 @@ ucs_status_t uct_ib_iface_create_qp(uct_ib_iface_t *iface,
 #endif
     if (qp == NULL) {
         uct_ib_check_memlock_limit_msg(
-                UCS_LOG_LEVEL_ERROR,
-                "iface=%p: failed to create %s QP "
-                "TX wr:%d sge:%d inl:%d resp:%d RX wr:%d sge:%d resp:%d: %m",
+                dev->ibv_context, UCS_LOG_LEVEL_ERROR,
+                "iface %p failed to create %s QP "
+                "TX wr:%d sge:%d inl:%d resp:%d RX wr:%d sge:%d resp:%d",
                 iface, uct_ib_qp_type_str(attr->qp_type), attr->cap.max_send_wr,
                 attr->cap.max_send_sge, attr->cap.max_inline_data,
                 attr->max_inl_cqe[UCT_IB_DIR_TX], attr->cap.max_recv_wr,
@@ -1101,10 +1121,10 @@ ucs_status_t uct_ib_iface_create_qp(uct_ib_iface_t *iface,
     attr->cap  = attr->ibv.cap;
     *qp_p      = qp;
 
-    ucs_debug("iface=%p: created %s QP 0x%x on %s:%d "
+    ucs_debug("%s: iface %p created %s QP 0x%x on %s:%d "
               "TX wr:%d sge:%d inl:%d resp:%d RX wr:%d sge:%d resp:%d",
-              iface, uct_ib_qp_type_str(attr->qp_type), qp->qp_num,
-              uct_ib_device_name(dev), iface->config.port_num,
+              uct_ib_device_name(dev), iface, uct_ib_qp_type_str(attr->qp_type),
+              qp->qp_num, uct_ib_device_name(dev), iface->config.port_num,
               attr->cap.max_send_wr, attr->cap.max_send_sge,
               attr->cap.max_inline_data, attr->max_inl_cqe[UCT_IB_DIR_TX],
               attr->cap.max_recv_wr, attr->cap.max_recv_sge,
@@ -1117,25 +1137,25 @@ ucs_status_t uct_ib_verbs_create_cq(uct_ib_iface_t *iface, uct_ib_dir_t dir,
                                     const uct_ib_iface_init_attr_t *init_attr,
                                     int preferred_cpu, size_t inl)
 {
-    uct_ib_device_t *dev = uct_ib_iface_device(iface);
-    unsigned cq_size     = uct_ib_cq_size(iface, init_attr, dir);
+    struct ibv_context *ibv_context = uct_ib_iface_device(iface)->ibv_context;
+    unsigned cq_size                = uct_ib_cq_size(iface, init_attr, dir);
     struct ibv_cq *cq;
 #if HAVE_DECL_IBV_CREATE_CQ_EX
     struct ibv_cq_init_attr_ex cq_attr = {};
 
     uct_ib_fill_cq_attr(&cq_attr, init_attr, iface, preferred_cpu, cq_size);
 
-    cq = ibv_cq_ex_to_cq(ibv_create_cq_ex(dev->ibv_context, &cq_attr));
+    cq = ibv_cq_ex_to_cq(ibv_create_cq_ex(ibv_context, &cq_attr));
     if (!cq && ((errno == EOPNOTSUPP) || (errno == ENOSYS)))
 #endif
     {
         iface->config.max_inl_cqe[dir] = 0;
-        cq = ibv_create_cq(dev->ibv_context, cq_size, NULL, iface->comp_channel,
+        cq = ibv_create_cq(ibv_context, cq_size, NULL, iface->comp_channel,
                            preferred_cpu);
     }
 
     if (cq == NULL) {
-        uct_ib_check_memlock_limit_msg(UCS_LOG_LEVEL_ERROR,
+        uct_ib_check_memlock_limit_msg(ibv_context, UCS_LOG_LEVEL_ERROR,
                                        "ibv_create_cq(cqe=%d)", cq_size);
         return UCS_ERR_IO_ERROR;
     }
@@ -1282,6 +1302,46 @@ out_mask_info_failed:
     return UCS_OK;
 }
 
+static unsigned uct_ib_iface_gid_index(uct_ib_iface_t *iface,
+                                       unsigned long cfg_gid_index)
+{
+    uct_ib_device_t *dev = uct_ib_iface_device(iface);
+    uint8_t port_num     = iface->config.port_num;
+    int gid_tbl_len      = uct_ib_device_port_attr(dev, port_num)->gid_tbl_len;
+    unsigned gid_index   = UCT_IB_DEVICE_DEFAULT_GID_INDEX;
+    uct_ib_device_gid_info_t gid_info;
+    ucs_status_t status;
+    uint32_t oui;
+
+    if (cfg_gid_index != UCS_ULUNITS_AUTO) {
+        return cfg_gid_index;
+    }
+
+    if (!iface->config.flid_enabled ||
+        (gid_tbl_len <= UCT_IB_DEVICE_ROUTABLE_FLID_GID_INDEX)) {
+        goto out;
+    }
+
+    status = uct_ib_device_query_gid_info(
+                          dev->ibv_context, uct_ib_device_name(dev), port_num,
+                          UCT_IB_DEVICE_ROUTABLE_FLID_GID_INDEX, &gid_info);
+    if (status != UCS_OK) {
+        goto out;
+    }
+
+    if (uct_ib_iface_gid_extract_flid(&gid_info.gid) == 0) {
+        goto out;
+    }
+
+    oui = be32toh(gid_info.gid.global.interface_id & 0xffffff) >> 8;
+    if (oui == UCT_IB_GUID_OPENIB_OUI) {
+        gid_index = UCT_IB_DEVICE_ROUTABLE_FLID_GID_INDEX;
+    }
+
+out:
+    return gid_index;
+}
+
 static ucs_status_t
 uct_ib_iface_init_gid_info(uct_ib_iface_t *iface,
                            const uct_ib_iface_config_t *config)
@@ -1303,10 +1363,8 @@ uct_ib_iface_init_gid_info(uct_ib_iface_t *iface,
             goto out;
         }
     } else {
-        gid_info->gid_index             = (cfg_gid_index ==
-                                           UCS_ULUNITS_AUTO) ?
-                                          UCT_IB_MD_DEFAULT_GID_INDEX :
-                                          cfg_gid_index;
+        gid_info->gid_index             = uct_ib_iface_gid_index(iface,
+                                                                 cfg_gid_index);
         gid_info->roce_info.ver         = UCT_IB_DEVICE_ROCE_ANY;
         gid_info->roce_info.addr_family = 0;
     }
@@ -1358,6 +1416,18 @@ uint8_t uct_ib_iface_config_select_sl(const uct_ib_iface_config_t *ib_config)
     return (uint8_t)ib_config->sl;
 }
 
+void uct_ib_iface_set_reverse_sl(uct_ib_iface_t *ib_iface,
+                                 const uct_ib_iface_config_t *ib_config)
+{
+    if (ib_config->reverse_sl == UCS_ULUNITS_AUTO) {
+        ib_iface->config.reverse_sl = ib_iface->config.sl;
+        return;
+    }
+
+    ucs_assert(ib_config->reverse_sl < UCT_IB_SL_NUM);
+    ib_iface->config.reverse_sl = (uint8_t)ib_config->reverse_sl;
+}
+
 UCS_CLASS_INIT_FUNC(uct_ib_iface_t, uct_iface_ops_t *tl_ops,
                     uct_ib_iface_ops_t *ops, uct_md_h md, uct_worker_h worker,
                     const uct_iface_params_t *params,
@@ -1420,12 +1490,15 @@ UCS_CLASS_INIT_FUNC(uct_ib_iface_t, uct_iface_ops_t *tl_ops,
     self->config.port_num           = port_num;
     /* initialize to invalid value */
     self->config.sl                 = UCT_IB_SL_NUM;
+    self->config.reverse_sl         = UCT_IB_SL_NUM;
     self->config.hop_limit          = config->hop_limit;
     self->release_desc.cb           = uct_ib_iface_release_desc;
     self->config.qp_type            = init_attr->qp_type;
     self->config.flid_enabled       = config->flid_enabled;
     uct_ib_iface_set_path_mtu(self, config);
 
+    self->config.send_overhead = config->send_overhead;
+
     if (ucs_derived_of(worker, uct_priv_worker_t)->thread_mode == UCS_THREAD_MODE_MULTI) {
         ucs_error("IB transports do not support multi-threaded worker");
         return UCS_ERR_INVALID_PARAM;
@@ -1694,13 +1767,15 @@ ucs_status_t uct_ib_iface_query(uct_ib_iface_t *iface, size_t xport_hdr_len,
 ucs_status_t
 uct_ib_iface_estimate_perf(uct_iface_h iface, uct_perf_attr_t *perf_attr)
 {
-    uct_ep_operation_t op  = UCT_ATTR_VALUE(PERF, perf_attr, operation,
-                                            OPERATION, UCT_EP_OP_LAST);
-    const double wqe_fetch = 350e-9;
-    double send_pre_overhead, send_post_overhead;
+    uct_ib_iface_t *ib_iface = ucs_derived_of(iface, uct_ib_iface_t);
+    uct_ep_operation_t op    = UCT_ATTR_VALUE(PERF, perf_attr, operation,
+                                              OPERATION, UCT_EP_OP_LAST);
+    const double wqe_fetch   = 350e-9;
+    double send_pre_overhead, send_pre_overhead_bcopy, send_post_overhead;
     double send_post_overhead_zcopy;
     uct_iface_attr_t iface_attr;
     ucs_status_t status;
+    uct_ib_iface_send_overhead_t *send_overhead;
 
     status = uct_iface_query(iface, &iface_attr);
     if (status != UCS_OK) {
@@ -1715,23 +1790,27 @@ uct_ib_iface_estimate_perf(uct_iface_h iface, uct_perf_attr_t *perf_attr)
         break;
     default:
         send_pre_overhead        = iface_attr.overhead;
-        /* Doorbell write effect on CPU operations pipeline */
         send_post_overhead       = 40e-9;
-        /* Completion for every operation */
         send_post_overhead_zcopy = 20e-9;
     }
+    send_pre_overhead_bcopy = 5e-9;
+    send_overhead           = &ib_iface->config.send_overhead;
 
     if (perf_attr->field_mask & UCT_PERF_ATTR_FIELD_SEND_PRE_OVERHEAD) {
-        perf_attr->send_pre_overhead = send_pre_overhead;
+        perf_attr->send_pre_overhead = ucs_time_units_to_sec(
+                send_overhead->wqe_post, send_pre_overhead);
         if (uct_ep_op_is_bcopy(op)) {
-            perf_attr->send_pre_overhead += 5e-9; /* Allocate send desc */
+            perf_attr->send_pre_overhead += ucs_time_units_to_sec(
+                    send_overhead->bcopy, send_pre_overhead_bcopy);
         }
     }
 
     if (perf_attr->field_mask & UCT_PERF_ATTR_FIELD_SEND_POST_OVERHEAD) {
-        perf_attr->send_post_overhead = send_post_overhead;
+        perf_attr->send_post_overhead =
+                ucs_time_units_to_sec(send_overhead->db, send_post_overhead);
         if (uct_ep_op_is_zcopy(op)) {
-            perf_attr->send_post_overhead += send_post_overhead_zcopy;
+            perf_attr->send_post_overhead += ucs_time_units_to_sec(
+                    send_overhead->cqe, send_post_overhead_zcopy);
         }
     }
 
@@ -1746,7 +1825,8 @@ uct_ib_iface_estimate_perf(uct_iface_h iface, uct_perf_attr_t *perf_attr)
     if (perf_attr->field_mask & UCT_PERF_ATTR_FIELD_LATENCY) {
         perf_attr->latency = iface_attr.latency;
         if (uct_ep_op_is_bcopy(op) || uct_ep_op_is_zcopy(op)) {
-            perf_attr->latency.c += wqe_fetch;
+            perf_attr->latency.c +=
+                    ucs_time_units_to_sec(send_overhead->wqe_fetch, wqe_fetch);
         }
     }
 
@@ -1754,6 +1834,10 @@ uct_ib_iface_estimate_perf(uct_iface_h iface, uct_perf_attr_t *perf_attr)
         perf_attr->max_inflight_eps = SIZE_MAX;
     }
 
+    if (perf_attr->field_mask & UCT_PERF_ATTR_FIELD_FLAGS) {
+        perf_attr->flags = 0;
+    }
+
     return UCS_OK;
 }
 
