@@ -14,6 +14,9 @@
 #include <ucs/time/time.h>
 
 
+#define REVERSE_SL_MASK   UCS_MASK(4)
+
+
 static UCS_F_ALWAYS_INLINE void
 uct_dc_mlx5_add_flush_remote(uct_dc_mlx5_ep_t *ep)
 {
@@ -36,6 +39,11 @@ uct_dc_mlx5_set_dgram_seg(uct_ib_mlx5_txwq_t *txwq,
     to_av->fl_mlid      = iface->tx.av_fl_mlid;
     to_av->rlid         = av->rlid;
 
+    /* Setting reverse_sl */
+    to_av->dqp_dct &= ~REVERSE_SL_MASK;
+    to_av->dqp_dct |= iface->super.super.super.config.reverse_sl &
+                      REVERSE_SL_MASK;
+
     return uct_ib_mlx5_set_dgram_seg_grh(seg, grh_av);
 }
 
@@ -1260,6 +1268,13 @@ UCS_CLASS_INIT_FUNC(uct_dc_mlx5_ep_t, uct_dc_mlx5_iface_t *iface,
         self->flush_rkey_hi = 0;
     }
 
+    if ((iface->super.super.config.max_rd_atomic == 16) !=
+        !!(if_addr->flags & UCT_DC_MLX5_IFACE_ADDR_MAX_RD_ATOMIC_16)) {
+        ucs_diag("max_rd_atomic values do not match on peers (local is %u), "
+                 "set UCX_DC_MLX5_MAX_RD_ATOMIC=16 to resolve this issue.",
+                 iface->super.super.config.max_rd_atomic);
+    }
+
     return uct_dc_mlx5_ep_basic_init(iface, self);
 }
 
