@@ -14,8 +14,11 @@ extern "C" {
 #include <ucp/core/ucp_rkey.h>
 #include <ucp/core/ucp_ep.inl>
 #include <ucp/dt/dt.h>
+#include <ucs/type/float8.h>
 }
 
+#include <cmath>
+
 class test_ucp_mmap : public ucp_test {
 public:
     test_ucp_mmap()
@@ -166,8 +169,8 @@ protected:
                               bool expect_rma_offload);
 
 private:
-    void expect_same_distance(const ucs_sys_dev_distance_t &dist1,
-                              const ucs_sys_dev_distance_t &dist2);
+    void check_distance_precision(double rkey_value, double topo_value,
+                                  size_t pack_min, size_t pack_max);
     void test_rkey_proto(ucp_mem_h memh);
     void test_rereg_local_mem(ucp_mem_h memh, void *ptr, size_t size,
                               unsigned map_flags);
@@ -349,18 +352,32 @@ void test_ucp_mmap::test_rkey_management(ucp_mem_h memh, bool is_dummy,
     ucp_rkey_buffer_release(rkey_buffer);
 }
 
-void test_ucp_mmap::expect_same_distance(const ucs_sys_dev_distance_t &dist1,
-                                         const ucs_sys_dev_distance_t &dist2)
+void test_ucp_mmap::check_distance_precision(double rkey_value,
+                                             double topo_value,
+                                             size_t pack_min,
+                                             size_t pack_max)
 {
     /* Expect the implementation to always provide a reasonable precision w.r.t.
      * real-world bandwidth and latency ballpark numbers.
      */
-    EXPECT_NEAR(dist1.bandwidth, dist2.bandwidth, 600e6); /* 600 MBs accuracy */
-    EXPECT_NEAR(dist1.latency, dist2.latency, 20e-9); /* 20 nsec accuracy */
+    double allowed_diff_ratio = 1 - UCS_FP8_PRECISION;
+
+    if (rkey_value == pack_min) {
+        /* Capped by pack_min, no cache entry */
+        EXPECT_LE(std::lround(topo_value), pack_min);
+    } else if (rkey_value == pack_max) {
+        /* Capped by pack_max, no cache entry */
+        EXPECT_GE(std::lround(topo_value), pack_max);
+    } else {
+        /* Inside the borders or cache entry */
+        EXPECT_NEAR(rkey_value, topo_value, topo_value * allowed_diff_ratio);
+    }
 }
 
 void test_ucp_mmap::test_rkey_proto(ucp_mem_h memh)
 {
+    ucs_sys_dev_distance_t rkey_dist, topo_dist;
+    ucs_sys_device_t sys_dev;
     ucs_status_t status;
 
     /* Detect system device of the allocated memory */
@@ -414,11 +431,15 @@ void test_ucp_mmap::test_rkey_proto(ucp_mem_h memh)
         /* Compare original system distance and unpacked rkey system distance */
         for (ucp_lane_index_t lane = 0; lane < ep_config->key.num_lanes;
              ++lane) {
-            ucs_sys_device_t sys_dev = ep_config->key.lanes[lane].dst_sys_dev;
-            expect_same_distance(rkey_config->lanes_distance[lane],
-                                 (sys_dev == UCS_SYS_DEVICE_ID_UNKNOWN) ?
-                                         ucs_topo_default_distance :
-                                         sys_distance[sys_dev]);
+            sys_dev   = ep_config->key.lanes[lane].dst_sys_dev;
+            rkey_dist = rkey_config->lanes_distance[lane];
+            topo_dist = (sys_dev == UCS_SYS_DEVICE_ID_UNKNOWN) ?
+                        ucs_topo_default_distance : sys_distance[sys_dev];
+
+            check_distance_precision(rkey_dist.bandwidth, topo_dist.bandwidth,
+                                     UCS_FP8_MIN_BW, UCS_FP8_MAX_BW);
+            check_distance_precision(rkey_dist.latency, topo_dist.latency,
+                                     UCS_FP8_MIN_LAT, UCS_FP8_MAX_LAT);
         }
     }
 
@@ -856,10 +877,7 @@ UCS_TEST_P(test_ucp_mmap, fixed) {
 
     for (int i = 0; i < 1000 / ucs::test_time_multiplier(); ++i) {
         size_t size = (i + 1) * ((i % 2) ? 1000 : 1);
-        void *ptr   = ucs::mmap_fixed_address(size);
-        if (ptr == nullptr) {
-            UCS_TEST_ABORT("mmap failed to allocate memory region");
-        }
+        ucs::mmap_fixed_address ptr(size);
 
         ucp_mem_h memh;
         ucp_mem_map_params_t params;
@@ -867,18 +885,19 @@ UCS_TEST_P(test_ucp_mmap, fixed) {
         params.field_mask = UCP_MEM_MAP_PARAM_FIELD_ADDRESS |
                             UCP_MEM_MAP_PARAM_FIELD_LENGTH |
                             UCP_MEM_MAP_PARAM_FIELD_FLAGS;
-        params.address    = ptr;
+        params.address    = *ptr;
         params.length     = size;
         params.flags      = UCP_MEM_MAP_FIXED | UCP_MEM_MAP_ALLOCATE;
 
         status = ucp_mem_map(sender().ucph(), &params, &memh);
         ASSERT_UCS_OK(status);
-        EXPECT_EQ(ucp_memh_address(memh), ptr);
+        EXPECT_EQ(ucp_memh_address(memh), *ptr);
         EXPECT_GE(ucp_memh_length(memh), size);
 
         is_dummy = (size == 0);
         test_rkey_management(memh, is_dummy, is_tl_rdma());
 
+        ptr.detach();
         status = ucp_mem_unmap(sender().ucph(), memh);
         ASSERT_UCS_OK(status);
     }
@@ -886,6 +905,80 @@ UCS_TEST_P(test_ucp_mmap, fixed) {
 
 UCP_INSTANTIATE_TEST_CASE_GPU_AWARE(test_ucp_mmap)
 
+class test_ucp_mmap_atomic : public test_ucp_mmap {
+public:
+    static void get_test_variants(std::vector<ucp_test_variant> &variants)
+    {
+        test_ucp_mmap::get_test_variants(variants,
+                                         UCP_FEATURE_TAG | UCP_FEATURE_AMO64);
+    }
+};
+
+/* Use a buffer for send/recv, and then reuse it for atomic operations */
+UCS_TEST_P(test_ucp_mmap_atomic, reuse_buffer)
+{
+    mem_buffer sbuf(UCS_MBYTE, UCS_MEMORY_TYPE_HOST, 1);
+    mem_buffer rbuf(UCS_MBYTE, UCS_MEMORY_TYPE_HOST);
+
+    /* Send/receive from buffers to trigger adding them to registration cache */
+    {
+        static constexpr uint64_t TAG = 0xdeadbeef;
+        ucp_request_param_t param;
+
+        param.op_attr_mask = 0;
+        auto sreq = ucp_tag_send_nbx(sender().ep(), sbuf.ptr(), sbuf.size(),
+                                     TAG, &param);
+        auto rreq = ucp_tag_recv_nbx(receiver().worker(), rbuf.ptr(),
+                                     rbuf.size(), TAG, 0, &param);
+
+        ASSERT_UCS_OK(requests_wait({sreq, rreq}));
+    }
+
+    /* Map the receive buffer for atomic operations */
+    ucp_mem_h memh;
+    ucp_rkey_h rkey;
+    {
+        ucp_mem_map_params_t params;
+        params.field_mask = UCP_MEM_MAP_PARAM_FIELD_ADDRESS |
+                            UCP_MEM_MAP_PARAM_FIELD_LENGTH |
+                            UCP_MEM_MAP_PARAM_FIELD_FLAGS;
+        params.address    = rbuf.ptr();
+        params.length     = rbuf.size();
+        params.flags      = mem_map_flags();
+
+        ASSERT_UCS_OK(ucp_mem_map(receiver().ucph(), &params, &memh));
+
+        void *rkey_buffer;
+        size_t rkey_size;
+        ASSERT_UCS_OK(ucp_rkey_pack(receiver().ucph(), memh, &rkey_buffer,
+                                    &rkey_size));
+        ASSERT_UCS_OK(ucp_ep_rkey_unpack(sender().ep(), rkey_buffer, &rkey));
+
+        ucp_rkey_buffer_release(rkey_buffer);
+    }
+
+    /* Perform atomic operation */
+    {
+        uint64_t value = 1;
+        ucp_request_param_t param;
+
+        param.op_attr_mask = UCP_OP_ATTR_FIELD_DATATYPE;
+        param.datatype     = ucp_dt_make_contig(sizeof(value));
+        auto sreq = ucp_atomic_op_nbx(sender().ep(), UCP_ATOMIC_OP_ADD, &value,
+                                      1, (uintptr_t)rbuf.ptr(), rkey, &param);
+
+        param.op_attr_mask = 0;
+        auto freq          = ucp_ep_flush_nbx(sender().ep(), &param);
+
+        ASSERT_UCS_OK(requests_wait({sreq, freq}));
+    }
+
+    /* Unmap the buffer */
+    ucp_rkey_destroy(rkey);
+    ASSERT_UCS_OK(ucp_mem_unmap(receiver().ucph(), memh));
+}
+
+UCP_INSTANTIATE_TEST_CASE_GPU_AWARE(test_ucp_mmap_atomic)
 
 class test_ucp_rkey_compare : public test_ucp_mmap {
 public:
