@@ -12,11 +12,11 @@
 #include <uct/api/v2/uct_v2.h>
 extern "C" {
 #include <ucs/time/time.h>
+#include <ucs/sys/ptr_arith.h>
 #include <ucs/sys/sys.h>
 #include <ucs/sys/string.h>
 #include <ucs/arch/bitops.h>
 #include <ucs/arch/atomic.h>
-#include <ucs/sys/math.h>
 #if HAVE_IB
 #include <uct/ib/base/ib_md.h>
 #endif
@@ -66,6 +66,9 @@ void* test_md::alloc_thread(void *arg)
 ucs_status_t test_md::reg_mem(unsigned flags, void *address, size_t length,
                               uct_mem_h *memh_p)
 {
+    /* Register memory respecting MD reg_alignment */
+    ucs_align_ptr_range(&address, &length, md_attr().reg_alignment);
+
     uct_md_mem_reg_params_t reg_params;
 
     reg_params.field_mask = UCT_MD_MEM_REG_FIELD_FLAGS;
@@ -101,7 +104,7 @@ void test_md::test_reg_mem(unsigned access_mask,
     params.flags       = UCT_MD_MEM_DEREG_FLAG_INVALIDATE;
     params.comp        = &comp().comp;
 
-    if (!is_supported_reg_mem_flags(access_mask)) {
+    if (!check_invalidate_support(access_mask)) {
         params.field_mask = UCT_MD_MEM_DEREG_FIELD_COMPLETION |
                             UCT_MD_MEM_DEREG_FIELD_FLAGS |
                             UCT_MD_MEM_DEREG_FIELD_MEMH;
@@ -173,7 +176,7 @@ test_md::test_md()
     /* coverity[uninit_member] */
 }
 
-bool test_md::is_supported_reg_mem_flags(unsigned reg_flags) const
+bool test_md::check_invalidate_support(unsigned reg_flags) const
 {
     return (reg_flags & md_flags_remote_rma) ?
            check_caps(UCT_MD_FLAG_INVALIDATE_RMA) :
@@ -273,12 +276,6 @@ void test_md::dereg_cb(uct_completion_t *comp)
     md_comp->self->m_comp_count++;
 }
 
-bool test_md::is_gpu_ipc() const
-{
-    return (GetParam().md_name == "cuda_ipc") ||
-	   (GetParam().md_name == "rocm_ipc");
-}
-
 UCS_TEST_SKIP_COND_P(test_md, rkey_ptr,
                      !check_caps(UCT_MD_FLAG_ALLOC |
                                  UCT_MD_FLAG_RKEY_PTR)) {
@@ -387,6 +384,7 @@ UCS_TEST_SKIP_COND_P(test_md, alloc,
     uct_md_h md_ref               = md();
     uct_alloc_method_t method     = UCT_ALLOC_METHOD_MD;
     unsigned num_alloc_failures   = 0;
+    std::vector<char> key(md_attr().rkey_packed_size);
     size_t size, orig_size;
     ucs_status_t status;
     void *address;
@@ -394,7 +392,6 @@ UCS_TEST_SKIP_COND_P(test_md, alloc,
     uct_allocated_memory_t mem;
     uct_mem_alloc_params_t alloc_params;
     uct_md_mkey_pack_params_t pack_params;
-    uint64_t key;
 
     alloc_params.field_mask = UCT_MEM_ALLOC_PARAM_FIELD_FLAGS |
                               UCT_MEM_ALLOC_PARAM_FIELD_ADDRESS |
@@ -446,9 +443,11 @@ UCS_TEST_SKIP_COND_P(test_md, alloc,
                 memset(address, 0xBB, size);
             }
 
-            status = uct_md_mkey_pack_v2(md(), mem.memh, address, size,
-                                         &pack_params, &key);
-            ASSERT_UCS_OK(status);
+            if (md_attr().rkey_packed_size != 0) {
+                status = uct_md_mkey_pack_v2(md(), mem.memh, address, size,
+                                             &pack_params, key.data());
+                ASSERT_UCS_OK(status);
+            }
 
             uct_mem_free(&mem);
         }
@@ -529,6 +528,8 @@ UCS_TEST_P(test_md, mem_type_detect_mds) {
 }
 
 UCS_TEST_P(test_md, mem_query) {
+    ASSERT_GT(md_attr().reg_alignment, 0);
+
     for (auto mem_type : mem_buffer::supported_mem_types()) {
         if (!(md_attr().detect_mem_types & UCS_BIT(mem_type))) {
             continue;
@@ -609,7 +610,7 @@ UCS_TEST_SKIP_COND_P(test_md, reg,
 
             alloc_memory(&address, size, &fill_buffer[0], mem_type);
 
-            status = uct_md_mem_reg(md(), address, size, UCT_MD_MEM_ACCESS_ALL, &memh);
+            status = reg_mem(UCT_MD_MEM_ACCESS_ALL, address, size, &memh);
 
             ASSERT_UCS_OK(status);
             ASSERT_TRUE(memh != UCT_MEM_HANDLE_NULL);
@@ -647,8 +648,7 @@ UCS_TEST_SKIP_COND_P(test_md, reg_perf,
             unsigned n = 0;
             while (n < count) {
                 uct_mem_h memh;
-                status = uct_md_mem_reg(md(), ptr, size, UCT_MD_MEM_ACCESS_ALL,
-                        &memh);
+                status = reg_mem(UCT_MD_MEM_ACCESS_ALL, ptr, size, &memh);
                 ASSERT_UCS_OK(status);
                 ASSERT_TRUE(memh != UCT_MEM_HANDLE_NULL);
 
@@ -836,8 +836,9 @@ UCS_TEST_P(test_md, sockaddr_accessibility) {
 /* This test registers region N times and later deregs it N/2 times and
  * invalidates N/2 times - mix multiple dereg and invalidate calls.
  * Guarantee that all packed keys are unique. */
-UCS_TEST_SKIP_COND_P(test_md, invalidate, !check_caps(UCT_MD_FLAG_INVALIDATE) ||
-		                                                    is_gpu_ipc())
+UCS_TEST_SKIP_COND_P(test_md, invalidate,
+                     !check_caps(UCT_MD_FLAG_INVALIDATE) ||
+                     !check_reg_mem_type(UCS_MEMORY_TYPE_HOST))
 {
     static const size_t size       = 1 * UCS_MBYTE;
     const int limit                = 64;
@@ -989,8 +990,8 @@ UCS_TEST_P(test_md, rkey_compare_params_check)
 }
 
 // SM case is covered by XPMEM which has registration capability
-UCS_TEST_SKIP_COND_P(test_md, rkey_compare, !check_caps(UCT_MD_FLAG_REG) ||
-		                                               is_gpu_ipc())
+UCS_TEST_SKIP_COND_P(test_md, rkey_compare,
+                     !check_reg_mem_type(UCS_MEMORY_TYPE_HOST))
 {
     size_t size                      = 4096;
     void *address                    = NULL;
@@ -1079,13 +1080,15 @@ UCS_TEST_SKIP_COND_P(test_md_fork, fork,
     ASSERT_EQ(pid, waitpid(pid, &child_status, 0));
     EXPECT_TRUE(WIFEXITED(child_status)) << ucs::exit_status_info(child_status);
 
+#ifndef __SANITIZE_ADDRESS__
     if (!RUNNING_ON_VALGRIND) {
-        /* Under valgrind, leaks are possible due to early exit, so don't expect
-         * an exit status of 0
+        /* Under valgrind or ASAN, leaks are possible due to early exit,
+         * so don't expect an exit status of 0
          */
         EXPECT_EQ(0, WEXITSTATUS(child_status)) <<
                 ucs::exit_status_info(child_status);
     }
+#endif
 
     free(page);
 }
@@ -1134,7 +1137,7 @@ class test_md_non_blocking : public test_md
 protected:
     void init() override {
         /* ODPv1 IB feature can work only for certain DEVX configuration */
-        modify_config("MLX5_DEVX_OBJECTS", "dct,dcsrq", IGNORE_IF_NOT_EXIST);
+        modify_config("IB_MLX5_DEVX_OBJECTS", "dct,dcsrq", IGNORE_IF_NOT_EXIST);
         test_md::init();
     }
 };
